@@ -30,13 +30,12 @@ const char code_syscall[] = {
 
 const int code_syscall_size = round_up(sizeof(code_syscall), sizeof(long));
 
-static inline void __check_code_syscall(void)
+static inline void __always_unused __check_code_syscall(void)
 {
 	BUILD_BUG_ON(sizeof(code_syscall) != BUILTIN_SYSCALL_SIZE);
 	BUILD_BUG_ON(!is_log2(sizeof(code_syscall)));
 }
 
-
 void parasite_setup_regs(unsigned long new_ip, void *stack, user_regs_struct_t *regs)
 {
 	regs->pc = new_ip;
