@@ -435,9 +435,8 @@ static int parasite_dump_tty(struct parasite_tty_args *args)
 	__tty_ioctl(TIOCGSID, args->sid);
 	__tty_ioctl(TIOCGPGRP, args->pgrp);
 	__tty_ioctl(TIOCGEXCL,	args->st_excl);
-	
-	if (args->type == TTY_TYPE_PTM ||
-	    args->type == TTY_TYPE_PTS) {
+
+	if (args->type == TTY_TYPE__PTY) {
 		__tty_ioctl(TIOCGPKT,	args->st_pckt);
 		__tty_ioctl(TIOCGPTLCK,	args->st_lock);
 	}
