@@ -740,6 +740,25 @@ static int wait_helpers(struct task_restore_args *task_args)
 	return 0;
 }
 
+static int lsm_set_label(struct task_restore_args *args)
+{
+	int ret = -1;
+
+	if (!args->lsm_profile)
+		return 0;
+
+	pr_info("restoring lsm profile %s\n", args->lsm_profile);
+
+	ret = sys_write(args->proc_attr_current, args->lsm_profile, args->lsm_profile_len);
+	sys_close(args->proc_attr_current);
+	if (ret < 0) {
+		pr_err("can't write lsm profile\n");
+		return -1;
+	}
+
+	return ret;
+}
+
 /*
  * The main routine to restore task via sigreturn.
  * This one is very special, we never return there
@@ -799,10 +818,7 @@ long __export_restore_task(struct task_restore_args *args)
 	for (i = 0; i < args->nr_vmas; i++) {
 		vma_entry = args->tgt_vmas + i;
 
-		if (!vma_entry_is(vma_entry, VMA_AREA_REGULAR))
-			continue;
-
-		if (!vma_priv(vma_entry))
+		if (!vma_entry_is_private(vma_entry))
 			continue;
 
 		if (vma_entry->end >= TASK_SIZE)
@@ -820,10 +836,7 @@ long __export_restore_task(struct task_restore_args *args)
 	for (i = args->nr_vmas - 1; i >= 0; i--) {
 		vma_entry = args->tgt_vmas + i;
 
-		if (!vma_entry_is(vma_entry, VMA_AREA_REGULAR))
-			continue;
-
-		if (!vma_priv(vma_entry))
+		if (!vma_entry_is_private(vma_entry))
 			continue;
 
 		if (vma_entry->start > TASK_SIZE)
@@ -846,7 +859,7 @@ long __export_restore_task(struct task_restore_args *args)
 		if (!vma_entry_is(vma_entry, VMA_AREA_REGULAR))
 			continue;
 
-		if (vma_priv(vma_entry))
+		if (vma_entry_is_private(vma_entry))
 			continue;
 
 		va = restore_mapping(vma_entry);
@@ -1166,6 +1179,11 @@ long __export_restore_task(struct task_restore_args *args)
 	ret = ret || restore_dumpable_flag(&args->mm);
 	ret = ret || restore_pdeath_sig(args->t);
 
+	if (lsm_set_label(args) < 0) {
+		pr_err("lsm_set_label failed\n");
+		goto core_restore_end;
+	}
+
 	futex_set_and_wake(&thread_inprogress, args->nr_threads);
 
 	restore_finish_stage(CR_STATE_RESTORE_CREDS);
