@@ -12,20 +12,17 @@
 #include "protobuf.h"
 #include "protobuf/utsns.pb-c.h"
 
-int dump_uts_ns(int ns_pid, int ns_id)
+int dump_uts_ns(int ns_id)
 {
-	int ret, img_fd;
+	int ret;
+	struct cr_img *img;
 	struct utsname ubuf;
 	UtsnsEntry ue = UTSNS_ENTRY__INIT;
 
-	img_fd = open_image(CR_FD_UTSNS, O_DUMP, ns_id);
-	if (img_fd < 0)
+	img = open_image(CR_FD_UTSNS, O_DUMP, ns_id);
+	if (!img)
 		return -1;
 
-	ret = switch_ns(ns_pid, &uts_ns_desc, NULL);
-	if (ret < 0)
-		goto err;
-
 	ret = uname(&ubuf);
 	if (ret < 0) {
 		pr_perror("Error calling uname");
@@ -35,27 +32,27 @@ int dump_uts_ns(int ns_pid, int ns_id)
 	ue.nodename = ubuf.nodename;
 	ue.domainname = ubuf.domainname;
 
-	ret = pb_write_one(img_fd, &ue, PB_UTSNS);
+	ret = pb_write_one(img, &ue, PB_UTSNS);
 err:
-	close(img_fd);
+	close_image(img);
 	return ret < 0 ? -1 : 0;
 }
 
 int prepare_utsns(int pid)
 {
-	int fd, ret;
+	int ret;
+	struct cr_img *img;
 	UtsnsEntry *ue;
-	struct sysctl_req req[3] = {
+	struct sysctl_req req[] = {
 		{ "kernel/hostname" },
 		{ "kernel/domainname" },
-		{ },
 	};
 
-	fd = open_image(CR_FD_UTSNS, O_RSTR, pid);
-	if (fd < 0)
+	img = open_image(CR_FD_UTSNS, O_RSTR, pid);
+	if (!img)
 		return -1;
 
-	ret = pb_read_one(fd, &ue, PB_UTSNS);
+	ret = pb_read_one(img, &ue, PB_UTSNS);
 	if (ret < 0)
 		goto out;
 
@@ -64,10 +61,10 @@ int prepare_utsns(int pid)
 	req[1].arg = ue->domainname;
 	req[1].type = CTL_STR(strlen(ue->domainname));
 
-	ret = sysctl_op(req, CTL_WRITE);
+	ret = sysctl_op(req, ARRAY_SIZE(req), CTL_WRITE);
 	utsns_entry__free_unpacked(ue, NULL);
 out:
-	close(fd);
+	close_image(img);
 	return ret;
 }
 
